@@ -122,60 +122,65 @@ func NewDefaultCompleter(opts ...Option) readline.AutoCompleter {
 		backslashCommands: []string{
 			`\!`,
 			`\?`,
+			`\C`,
+			`\H`,
+			`\T`,
+			`\Z`,
 			`\a`,
 			`\begin`,
+			`\bind`,
 			`\c`,
-			`\connect`,
-			`\C`,
 			`\cd`,
 			`\commit`,
+			`\connect`,
 			`\conninfo`,
-			`\copyright`,
 			`\copy`,
+			`\copyright`,
+			`\cset`,
 			`\d+`,
-			`\da+`,
-			`\da`,
-			`\daS+`,
-			`\daS`,
-			`\df+`,
-			`\df`,
-			`\dfS+`,
-			`\dfS`,
-			`\di+`,
-			`\di`,
-			`\diS+`,
-			`\diS`,
-			`\dm+`,
-			`\dm`,
-			`\dmS+`,
-			`\dmS`,
-			`\dn+`,
-			`\dn`,
-			`\dnS+`,
-			`\dnS`,
-			`\drivers`,
-			`\ds+`,
-			`\ds`,
 			`\dS+`,
 			`\dS`,
+			`\da+`,
+			`\daS+`,
+			`\daS`,
+			`\da`,
+			`\df+`,
+			`\dfS+`,
+			`\dfS`,
+			`\df`,
+			`\di+`,
+			`\diS+`,
+			`\diS`,
+			`\di`,
+			`\dm+`,
+			`\dmS+`,
+			`\dmS`,
+			`\dm`,
+			`\dn+`,
+			`\dnS+`,
+			`\dnS`,
+			`\dn`,
+			`\drivers`,
+			`\ds+`,
 			`\dsS+`,
 			`\dsS`,
+			`\ds`,
 			`\dt+`,
-			`\dt`,
 			`\dtS+`,
 			`\dtS`,
+			`\dt`,
 			`\dv+`,
-			`\dv`,
 			`\dvS+`,
 			`\dvS`,
+			`\dv`,
 			`\e`,
 			`\echo`,
 			`\f`,
 			`\g`,
+			`\getenv`,
 			`\gexec`,
 			`\gset`,
 			`\gx`,
-			`\H`,
 			`\i`,
 			`\ir`,
 			`\l+`,
@@ -191,13 +196,11 @@ func NewDefaultCompleter(opts ...Option) readline.AutoCompleter {
 			`\set`,
 			`\setenv`,
 			`\t`,
-			`\T`,
 			`\timing`,
 			`\unset`,
 			`\w`,
 			`\watch`,
 			`\x`,
-			`\Z`,
 		},
 	}
 	for _, o := range opts {
