@@ -28,14 +28,18 @@ import (
 	"github.com/alecthomas/chroma/v2"
 	"github.com/alecthomas/chroma/v2/formatters"
 	"github.com/alecthomas/chroma/v2/styles"
+	"github.com/go-git/go-billy/v5"
 	"github.com/xo/dburl"
 	"github.com/xo/dburl/passfile"
+	"github.com/xo/echartsgoja"
+	"github.com/xo/resvg"
 	"github.com/xo/tblfmt"
 	"github.com/xo/usql/drivers"
 	"github.com/xo/usql/drivers/completer"
 	"github.com/xo/usql/drivers/metadata"
 	"github.com/xo/usql/env"
 	"github.com/xo/usql/metacmd"
+	"github.com/xo/usql/metacmd/charts"
 	"github.com/xo/usql/rline"
 	"github.com/xo/usql/stmt"
 	ustyles "github.com/xo/usql/styles"
@@ -53,7 +57,11 @@ import (
 type Handler struct {
 	l    rline.IO
 	user *user.User
-	wd   string
+	// wd is the working directoyr.
+	wd string
+	// charts is the charts filesystem.
+	charts billy.Filesystem
+	// nopw indicates not asking for password.
 	nopw bool
 	// timing of every command executed
 	timing bool
@@ -79,7 +87,7 @@ type Handler struct {
 }
 
 // New creates a new input handler.
-func New(l rline.IO, user *user.User, wd string, nopw bool) *Handler {
+func New(l rline.IO, user *user.User, wd string, charts billy.Filesystem, nopw bool) *Handler {
 	f, iactive := l.Next, l.Interactive()
 	if iactive {
 		f = func() ([]rune, error) {
@@ -94,11 +102,12 @@ func New(l rline.IO, user *user.User, wd string, nopw bool) *Handler {
 		}
 	}
 	h := &Handler{
-		l:    l,
-		user: user,
-		wd:   wd,
-		nopw: nopw,
-		buf:  stmt.New(f),
+		l:      l,
+		user:   user,
+		wd:     wd,
+		charts: charts,
+		nopw:   nopw,
+		buf:    stmt.New(f),
 	}
 	if iactive {
 		l.SetOutput(h.outputHighlighter)
@@ -429,6 +438,8 @@ func (h *Handler) Execute(ctx context.Context, w io.Writer, opt metacmd.Option,
 		f = h.doExecSet
 	case metacmd.ExecWatch:
 		f = h.doExecWatch
+	case metacmd.ExecChart:
+		f = h.doExecChart
 	}
 	if err = drivers.WrapErr(h.u.Driver, f(ctx, w, opt, prefix, sqlstr, qtyp, bind)); err != nil {
 		if forceTrans {
@@ -1003,17 +1014,17 @@ func (h *Handler) Version(ctx context.Context) error {
 }
 
 // Print formats according to a format specifier and writes to handler's standard output.
-func (h *Handler) Print(format string, a ...interface{}) {
+func (h *Handler) Print(s string, v ...interface{}) {
 	if env.Get("QUIET") == "on" {
 		return
 	}
-	fmt.Fprintln(h.l.Stdout(), fmt.Sprintf(format, a...))
+	fmt.Fprintln(h.l.Stdout(), fmt.Sprintf(s, v...))
 }
 
 // doExecWatch repeatedly executes a query against the database.
 func (h *Handler) doExecWatch(ctx context.Context, w io.Writer, opt metacmd.Option, prefix, sqlstr string, qtyp bool, bind []interface{}) error {
 	for {
-		// this is the actual output that psql has: "Mon Jan 2006 3:04:05 PM MST"
+		// the actual output that psql has: "Mon Jan 2006 3:04:05 PM MST" -- which is _slightly_ different than RFC1123
 		// fmt.Fprintf(w, "%s (every %fs)\n\n", time.Now().Format("Mon Jan 2006 3:04:05 PM MST"), float64(opt.Watch)/float64(time.Second))
 		fmt.Fprintf(w, "%s (every %v)\n", time.Now().Format(time.RFC1123), opt.Watch)
 		fmt.Fprintln(w)
@@ -1031,6 +1042,82 @@ func (h *Handler) doExecWatch(ctx context.Context, w io.Writer, opt metacmd.Opti
 	}
 }
 
+// doExecChart executes a single query against the database, displaying its output as a chart.
+func (h *Handler) doExecChart(ctx context.Context, w io.Writer, opt metacmd.Option, prefix, sqlstr string, qtyp bool, bind []interface{}) error {
+	stdout, _, _ := h.l.Stdout(), h.l.Stderr(), h.l.Interactive()
+	typ := env.TermGraphics()
+	if !typ.Available() {
+		return text.ErrGraphicsNotSupported
+	}
+	if _, ok := opt.Params["help"]; ok {
+		fmt.Fprintln(stdout, text.ChartUsage)
+		return nil
+	}
+	cfg, err := charts.ParseArgs(opt.Params)
+	if err != nil {
+		return err
+	}
+	start := time.Now()
+	// query
+	rows, err := h.DB().QueryContext(ctx, sqlstr, bind...)
+	if err != nil {
+		return err
+	}
+	// get cols
+	cols, err := drivers.Columns(h.u, rows)
+	if err != nil {
+		return err
+	}
+	// process row(s)
+	transposed := make([][]string, len(cols))
+	clen, tfmt := len(cols), env.GoTime()
+	for rows.Next() {
+		row, err := h.scan(rows, clen, tfmt)
+		if err != nil {
+			return err
+		}
+		for i := range row {
+			transposed[i] = append(transposed[i], row[i])
+		}
+	}
+	// display
+	c, err := charts.MakeChart(cfg, cols, transposed)
+	if err != nil {
+		return err
+	}
+	data, err := c.ToEcharts()
+	if err != nil {
+		return err
+	}
+	echarts := echartsgoja.New(echartsgoja.WithWidthHeight(cfg.W, cfg.H))
+	res, err := echarts.RenderOptions(ctx, data)
+	if err != nil {
+		return err
+	}
+	if cfg.File != "" {
+		fmt.Println("writing to", cfg.File)
+		return os.WriteFile(cfg.File, []byte(res), 0o644)
+	}
+	img, err := resvg.Render([]byte(res), resvg.WithBackground(cfg.Background))
+	if err != nil {
+		return err
+	}
+	if err := typ.Encode(stdout, img); err != nil {
+		return err
+	}
+	if h.timing {
+		d := time.Since(start)
+		s := text.TimingDesc
+		v := []interface{}{float64(d.Microseconds()) / 1000}
+		if d > 1*time.Second {
+			s += " (%v)"
+			v = append(v, d.Round(1*time.Millisecond))
+		}
+		fmt.Fprintln(h.l.Stdout(), fmt.Sprintf(s, v...))
+	}
+	return nil
+}
+
 // doExecSingle executes a single query against the database based on its query type.
 func (h *Handler) doExecSingle(ctx context.Context, w io.Writer, opt metacmd.Option, prefix, sqlstr string, qtyp bool, bind []interface{}) error {
 	// exec or query
@@ -1158,12 +1245,11 @@ func (h *Handler) doQuery(ctx context.Context, w io.Writer, opt metacmd.Option,
 	case drivers.UseColumnTypes(h.u):
 		extra = append(extra, tblfmt.WithUseColumnTypes(true))
 	}
-	// wrap query with crosstab
 	resultSet := tblfmt.ResultSet(rows)
+	// wrap query with crosstab
 	if opt.Exec == metacmd.ExecCrosstab {
 		var err error
-		resultSet, err = tblfmt.NewCrosstabView(rows, append(extra, tblfmt.WithParams(opt.Crosstab...))...)
-		if err != nil {
+		if resultSet, err = tblfmt.NewCrosstabView(rows, append(extra, tblfmt.WithParams(opt.Crosstab...))...); err != nil {
 			return err
 		}
 		extra = nil
@@ -1401,7 +1487,7 @@ func (h *Handler) Include(path string, relative bool) error {
 		Err: h.l.Stderr(),
 		Pw:  h.l.Password,
 	}
-	p := New(l, h.user, filepath.Dir(path), h.nopw)
+	p := New(l, h.user, filepath.Dir(path), h.charts, h.nopw)
 	p.db, p.u = h.db, h.u
 	drivers.ConfigStmt(p.u, p.buf)
 	err = p.Run()
@@ -1433,6 +1519,9 @@ func (h *Handler) SetOutput(o io.WriteCloser) {
 	h.out = o
 }
 
+// FS is the filesystem interface.
+type FS interface{}
+
 // Error wraps handler errors.
 type Error struct {
 	Buf string
