@@ -1,4 +1,4 @@
-// Package metacmd contains meta information and implentation for usql's
+// Package metacmd contains meta information and implementation for usql's
 // backslash (\) commands.
 package metacmd
 
@@ -40,6 +40,8 @@ const (
 	Copyright
 	// Connect is the connect meta command (\c, \connect).
 	Connect
+	// SetConnVar is the set conn var command (\cset).
+	SetConnVar
 	// Copy is the copy meta command (\copy).
 	Copy
 	// Disconnect is the disconnect meta command (\Z).
@@ -54,6 +56,8 @@ const (
 	Describe
 	// Exec is the execute meta command (\g and variants).
 	Exec
+	// Bind is the bind meta command (\bind).
+	Bind
 	// Edit is the edit query buffer meta command (\e).
 	Edit
 	// Print is the print query buffer meta command (\p, \print, \raw).
@@ -66,6 +70,8 @@ const (
 	Write
 	// ChangeDir is the system change directory meta command (\cd).
 	ChangeDir
+	// GetEnv is the system get environment variable meta command (\getenv).
+	GetEnv
 	// SetEnv is the system set environment variable meta command (\setenv).
 	SetEnv
 	// Shell is the system shell exec meta command (\!).
@@ -82,8 +88,8 @@ const (
 	SetVar
 	// Unset is the variable unset meta command (\unset).
 	Unset
-	// SetFormatVar is the set format variable meta commands (\pset, \a, \C, \f, \H, \t, \T, \x).
-	SetFormatVar
+	// SetPrintVar is the set print variable meta commands (\pset, \a, \C, \f, \H, \t, \T, \x).
+	SetPrintVar
 	// Timing is the timing meta command (\timing).
 	Timing
 	// Stats is the show stats meta command (\ss and variants).
